package me.lensfrex.trailblazer.api.v1.controllers.auth.register;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;
import me.lensfrex.trailblazer.api.v1.beans.requests.RegisterRequestBody;
import me.lensfrex.trailblazer.api.v1.beans.responses.general.ResponseCode;
import me.lensfrex.trailblazer.api.v1.beans.responses.general.ResponseBase;
import me.lensfrex.trailblazer.api.v1.exceptions.RequestDataInvalidException;
import me.lensfrex.trailblazer.api.v1.exceptions.user.UserNameAlreadyExistsException;
import me.lensfrex.trailblazer.api.v1.service.auth.register.RegisterService;
import me.lensfrex.trailblazer.api.v1.utils.InputChecker;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
@RequestMapping("/user")
public class Register {
    @Resource
    private RegisterService registerService;

    private static final Gson gson = new Gson();

    @PostMapping(value = "/register", produces = "application/json")
    public String register(@RequestBody String request) {
        try {
            RegisterRequestBody registerRequestBody = gson.fromJson(request, RegisterRequestBody.class);

            if (registerRequestBody == null ||
                    InputChecker.hasInvalidChar(registerRequestBody.getUserName()) ||
                    InputChecker.hasInvisibleChar(registerRequestBody.getPassword())) {

                throw new RequestDataInvalidException();
            }

            return gson.toJson(ResponseBase.success(registerService.register(registerRequestBody)));
        } catch (JsonParseException | RequestDataInvalidException e) {
            return gson.toJson(ResponseBase.error(ResponseCode.REQUEST_FORMAT_INVALID, "请求的数据不正确"));
        } catch (UserNameAlreadyExistsException e) {
            return gson.toJson(ResponseBase.error(ResponseCode.USER_ALREADY_EXISTS, "用户名已经被使用"));
        } catch (Exception e) {
            return gson.toJson(ResponseBase.error(ResponseCode.SERVER_ERROR, "服务器程序发生错误，有个家伙又写bug了。Error:" + e.getMessage()));
        }
    }
}