package me.lensfrex.trailblazer.api.v1.service.auth.register;

import me.lensfrex.trailblazer.api.v1.beans.requests.RegisterRequestBody;
import me.lensfrex.trailblazer.api.v1.beans.responses.RegisterResponseData;
import me.lensfrex.trailblazer.api.v1.dao.UserDao;
import me.lensfrex.trailblazer.api.v1.exceptions.user.UserNameAlreadyExistsException;
import me.lensfrex.trailblazer.api.v1.utils.jwt.JWTManager;
import org.mindrot.jbcrypt.BCrypt;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.UUID;

@Service
public class RegisterService {
    private static final JWTManager jwtManager = JWTManager.getInstance();

    public RegisterResponseData register(RegisterRequestBody registerRequestBody) throws UserNameAlreadyExistsException {

        if (UserDao.isUserAlreadyExist(registerRequestBody.getUserName())) {
            throw new UserNameAlreadyExistsException("用户名已经被使用");
        }

        String userUUID = UUID.randomUUID().toString();
        String userBcryptPasswd = BCrypt.hashpw(registerRequestBody.getPassword(), BCrypt.gensalt());

        int newUid = UserDao.addUser(userUUID, registerRequestBody.getUserName(), userBcryptPasswd);

        Date expireDate = Date.from(Instant.now().plus(JWTManager.TOKEN_DEFAULT_EXPIRE_DAY, ChronoUnit.DAYS));

        return new RegisterResponseData(
                newUid,
                userUUID,
                jwtManager.createNewJWT(registerRequestBody.getUserName(), userUUID, newUid, expireDate),
                expireDate.getTime());
    }
}
